__author__ = "Jean-Pierre LESUEUR (@DarkCoderSc)"
__maintainer__ = "Jean-Pierre LESUEUR"
__email__ = "jplesueur@phrozen.io"
__copyright__ = "Copyright 2024, Phrozen"
__license__ = "Apache License 2.0"

from .tangeant_universe import TangentUniverse

__all__ = [
    'TangentUniverse',
]
